==> qryq-0.0.8.tgz: package/doco/present/markdown/present.md <==
# qryq

## stop RESTing, start using query queues

<sup>
  /ˈkwərik/
</sup>

### Brendan Graetz

![Brendan Graetz](https://si0.twimg.com/profile_images/2697395233/8dec79da8f5963a4cad4da0ebd6a532d.png "Brendan Graetz on Twitter")

[@bguiz](http://bguiz.com "Brendan Graetz on Twitter")

[bguiz.com](http://bguiz.com "Brendan Graetz")

----

## In one sentence

`qryq` is a NodeJs library that allows one to express a series of queries and define dependencies between them either in parallel, in sequence, or in a directed acyclic graph.

![Directed Acyclic Graph](http://upload.wikimedia.org/wikipedia/commons/thumb/3/39/Directed_acyclic_graph_3.svg/356px-Directed_acyclic_graph_3.svg.png)

---

<blockquote>
A directed graph with no directed cycles.<br />
That is, it is formed by a collection of vertices and directed edges, each edge connecting one vertex to another, such that there is no way to start at some vertex v and follow a sequence of edges that eventually loops back to v again
</blockquote>

-- [Directed acyclic graph](http://en.wikipedia.org/wiki/Directed_acyclic_graph)

----

## Implementation

- [node.js](http://nodejs.org)
- [Q](https://github.com/kriskowal/q)
- [underscore.js](http://underscorejs.org)
- [nodeunit](https://github.com/caolan/nodeunit)

----

## Inspiration

- Neil Jenkin's talk Tips, Tricks and Hacks in the Pursuit of Speed
  - [REST is slow](http://nmjenkins.com/presentations/network-speed.html#/14)
  - [Concatenate requests](http://nmjenkins.com/presentations/network-speed.html#/15)
  - [Concatenate responses](http://nmjenkins.com/presentations/network-speed.html#/16)

---

<pre>
  <code class="js">
POST /api/

[
    [ 'deleteMessages', {
        idList: [ 'msg1' ]
    }],
    [ 'getMailboxMessageList', {
        mailboxName: 'Inbox',
        position: 0,
        limit: 30,
        sort: 'date descending'
    }]
]
  </code>
</pre>

---

## Inspiration

- [Play framework](http://playframework.com)'s
  - [Linkedin talk by Yevgeniy Brikman](http://www.slideshare.net/brikis98/the-play-framework-at-linkedin)
  - See slides 85 through 88

---

<div>
  <iframe src="http://www.slideshare.net/slideshow/embed_code/22423382?rel=0&startSlide=85" width="512" height="421" frameborder="0" marginwidth="0" marginheight="0" scrolling="no" style="border:1px solid #CCC;border-width:1px 1px 0;margin-bottom:5px" allowfullscreen webkitallowfullscreen mozallowfullscreen>
  </iframe>
  <div style="margin-bottom:5px">
    <strong> <a href="http://www.slideshare.net/brikis98/the-play-framework-at-linkedin" title="The Play Framework at LinkedIn" target="_blank">The Play Framework at LinkedIn</a></strong> from <strong><a href="http://www.slideshare.net/brikis98" target="_blank">Yevgeniy Brikman</a></strong>
  </div>
</div>

----

## The Itch

- NodeJs callback spaghetti
- Fix this using promises
- While better, if the code is sufficiently complex, you can still end up with:
- Promise spaghetti

![Callback Spaghetti](image/callback-spaghetti.png)

----

### Light Bulb

[Question on S/O](http://stackoverflow.com/questions/17342401/q-executing-a-series-of-promises-and-defining-dependencies-between-them-in-a-d "Q - executing a series of promises and defining dependencies between them in a DAG")
![Stackoverflow Question](image/stackoverflow-qn.png)

----

## The Query Queue

<pre>
  <code>
[
  {id: "q1", api: "add", qry:{a:1, b:9}},
  {id: "q2", api: "add", qry:{a:99, b:1}},
  {id: "q3", api: "multiply", qry:{a: "#{q1}", b: "#{q2}"}},
  {id: "q4", api: "multiply", qry:{a: "#{q3}", b:5}}
]
  </code>
</pre>

<pre>
  <code>
`q2`                          --> add(99, 1)          --> 100
`q1`                          --> add(1, 9)           --> 10
`q3` --> multiply(`q1`, `q2`) --> multiply(10, 100)   --> 1000
`q4` --> multiply(`q3`, 5)    --> multiply(1000, 5)   --> 5000
  </code>
</pre>

- `q1` and `q2` may execute in any order,
- `q3` may only execute after *both* `q1` and `q2`,
- and `q4` executes last.
- Wiring is done automatically by `qryq`

---

### What about async?

- Dev productivity in accomplishing the same thing:
  - Sequential: ~Same
  - Parallel: ~Same
  - Dependent: A lot easier

----

## Real World Query Queue

<pre>
  <code>
[
  {"id":"qGeocodeOrigin","depends":[],"api":"gmapsGeoLookup","qry":{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805"}},
  {"id":"qGeocodeDestination","depends":[],"api":"gmapsGeoLookup","qry":{"address":"19 Bourke Street, Melbourne, VIC 3000"}},
  {"id":"qScore","depends":["qGeocodeOrigin","qGeocodeDestination"],"api":"score","qry":{
      "origin":{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805","lat":"#{qGeocodeOrigin}.lat","lon":"#{qGeocodeOrigin}.lon"},
      "journeyPlanner":"melbtrans",
      "destinations":[
        {
          "fixed":true,"class":"work","weight":0.8,
          "location":{"address":"19 Bourke Street, Melbourne, VIC 3000","lat":"#{qGeocodeDestination}.lat","lon":"#{qGeocodeDestination}.lon"},
          "modes":[{"form":"transit","max":{"time":2400}}]
        }
      ]
    }
  }
]
  </code>
</pre>

From [`walkre`](https://github.com/bguiz/walkre "walkre")

----

## Benefits

- Developer productivity
- Bandwidth & latency savings

---

### Benefits - Dev Productivity

<blockquote>
This is the Unix philosophy: Write programs that do one thing and do it well. Write programs to work together.
</blockquote>

-- Doug McIlroy, invented pipes in UNIX

---

### Benefits - Dev Productivity

- Less need to write dedicated API endpoints
  - instead write small API endpoints, and chain them together
  - readable && composable
  - declarative query rather than imperative
  - avoid callback spaghetti && promise spaghetti

---

### Benefits - Dev Productivity


- Less duplication of biz logic required
  - Client/ server
- Groups several queries together as an atomic unit
  - [Asynchronous UIs](http://blog.alexmaccaw.com/asynchronous-ui)

---

### Benefits - 'Net Traffic

- Concatenation of
  - Multiple requests
  - Multiple responses
- [Protocol overhead](http://sd.wareonearth.com/~phil/net/overhead/) minimised

----

## Limitations

- non-RESTful
- Testability?
- Expression engine

---

### Limitations - REST

- Do you need REST when you are *not* doing CRUD?
- What if you typically chain more than one CRUD operation together?

---

### Limitations - Testing

- Harder because clients may compose API calls in novel ways
- Need to write more robust code

---

### Limitations - Expressions

<blockquote>
`#{previousQry}.flights.length`
</blockquote>

- A `qry` references results of another `qry` in the same `qryq`
- Limited: Can only "drill down" through properties

----

Avoiding

- Callback spaghetti
- Promise spaghetti

---

- Request made by the client
- Before - 3 separate queries:

```
to gmapsGeoLookup:
{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805"}
{"address":"19 Bourke Street, Melbourne, VIC 3000"}
```

```
to score:
{
  "origin":{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805","lat":"123.999","lon":"456.999"},
  "journeyPlanner":"melbtrans",
  "destinations":[
    {
      "fixed":true,"class":"work","weight":0.8,"location":{"address":"19 Bourke Street, Melbourne, VIC 3000","lat":"789.111","lon":"123.111"},
      "modes":[{"form":"transit","max":{"time":2400}}]
    }
  ]
}
```

---

- Request made by the client
- After - a single query:

```
[
  {"id":"qGeocodeOrigin","depends":[],"api":"gmapsGeoLookup","qry":{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805"}},
  {"id":"qGeocodeDestination","depends":[],"api":"gmapsGeoLookup","qry":{"address":"19 Bourke Street, Melbourne, VIC 3000"}},
  {"id":"qScore","depends":["qGeocodeOrigin","qGeocodeDestination"],"api":"score","qry":{
      "origin":{"address":"36 Meadow Wood Walk, Narre Warren VIC 3805","lat":"#{qGeocodeOrigin}.lat","lon":"#{qGeocodeOrigin}.lon"},
      "journeyPlanner":"melbtrans",
      "destinations":[
        {
          "fixed":true,"class":"work","weight":0.8,
          "location":{"address":"19 Bourke Street, Melbourne, VIC 3000","lat":"#{qGeocodeDestination}.lat","lon":"#{qGeocodeDestination}.lon"},
          "modes":[{"form":"transit","max":{"time":2400}}]
        }
      ]
    }
  }
]
```

---

- ... however, this means that the code on the server can focus on just one thing
- similar in concept to the UNIX command line philosophy
  - make sure you do just one thing well, and
  - chain well with others

---

- Server `score` API before:
- Demonstrates how *not* to use promises

```javascript
exports.score = function(deferred, qry) {
  /* validate qry */
  qry.journeyPlanner = qry.journeyPlanner || 'gmaps';
  var needGeo =
    (qry.journeyPlanner === 'melbtrans')
    || _.some(qry.destinations, function(destination) {
      destination.hasOwnProperty('fixed') && (destination.fixed === true);
    });
  var originPromises = [];
  var destinationPromises = [];
  if (needGeo) {
    //get geolocations for all locations present
    if (!qry.origin.lat || !qry.origin.lon) {
      var originGeoDeferred = Q.defer();
      exports.gmapsGeoLookup(originGeoDeferred, qry.origin);
      originPromises.push(originGeoDeferred.promise);
    }
    _.each(qry.destinations, function(destination) {
      var destGeoDeferred = Q.defer();
      exports.gmapsGeoLookup(destGeoDeferred, destination.location);
      destinationPromises.push(destGeoDeferred.promise);
    }, this);
  }
  var originPromisesDeferred = Q.defer();
  Q.allSettled(originPromises).then(function(results) {
    _.each(results, function(result) {
      if (result.state === 'fulfilled') {
        var val = result.value;
        qry.origin.lat = val.lat;
        qry.origin.lon = val.lon;
      }
      else {
        console.log('originPromises allSettled:', result.error);
      }
    }, this);
    originPromisesDeferred.resolve(qry.origin);
  });
  var destinationsPromisesDeferred = Q.defer();
  Q.allSettled(destinationPromises).then(function(results) {
    _.each(results, function(result, idx) {
      if (result.state === 'fulfilled') {
        var val = result.value;
        qry.destinations[idx].location.lat = val.lat;
        qry.destinations[idx].location.lon = val.lon;
      }
      else {
        console.log('destinationPromises allSettled:', result.error);
      }
    }, this);
    destinationsPromisesDeferred.resolve(qry.destinations);
  });
  Q.allSettled([originPromisesDeferred.promise, destinationsPromisesDeferred.promise]).then(function(results) {
    //we don't care about the results returned, because they were modified in place in the qry object
    //more importantly, we are now assured that all addresses have a lat and lon, if needGeo is true
    var scorePromises = [];

    //get the transport information from the origin to each destination using each transport mode
    var origin = qry.origin;
    _.each(qry.destinations, function(destination) {
      //TODO check that weights add up for destinations
      _.each(destination.modes, function(mode) {
        //we have origin, destination, and mode
        //TODO check that weights add up for modes
        //now work out the transport information between this origin and this destination using this mode
        var scoreDeferred = Q.defer();
        scorePromises.push(scoreDeferred.promise);
        exports.scoreOne(scoreDeferred, {
          origin: origin,
          destination: destination.location,
          mode: mode,
          journeyPlanner: qry.journeyPlanner
        });
      });
    }, this);

    Q.allSettled(scorePromises).then(function(scoreResults) {
      var orig_dest_mode = {};
      _.each(scoreResults, function(result) {
        if (result.state === 'fulfilled') {
          var score = result.value;
          orig_dest_mode[score.origin.address] = 
            orig_dest_mode[score.origin.address] || 
            {};
          orig_dest_mode[score.origin.address][score.destination.address] = 
            orig_dest_mode[score.origin.address][score.destination.address] || 
            {};
          orig_dest_mode[score.origin.address][score.destination.address][score.mode.form] = 
            orig_dest_mode[score.origin.address][score.destination.address][score.mode.form] || 
            {
              origin: score.origin,
              destination: score.destination,
              mode: score.mode,
              score: score.score
            };
        }
        else {
          console.log('scorePromises allSettled:', result.error);
        }
      });

      //some business logic
      var out = {
        /* ... */
      };
      deferred.resolve(out);
    });
  });
};
```

---

- Server `score` API after:
- Demonstrates how to do the same thing, using query queues
- ~Half the number of lines of code required

```javascript
exports.score = function(deferred, qry) {
  /* validate qry */
  qry.journeyPlanner = qry.journeyPlanner || 'gmaps';

  var scorePromises = [];
  //get the transport information from the origin to each destination using each transport mode
  var origin = qry.origin;
  _.each(qry.destinations, function(destination) {
    //TODO check that weights add up for destinations
    _.each(destination.modes, function(mode) {
      //we have origin, destination, and mode
      //TODO check that weights add up for modes
      //now work out the transport information between this origin and this destination using this mode
      var scoreDeferred = Q.defer();
      scorePromises.push(scoreDeferred.promise);
      exports.scoreOne(scoreDeferred, {
        origin: origin,
        destination: destination.location,
        mode: mode,
        journeyPlanner: qry.journeyPlanner
      });
    });
  });

  Q.allSettled(scorePromises).then(function(scoreResults) {
    var orig_dest_mode = {};
    _.each(scoreResults, function(result) {
      if (result.state === 'fulfilled') {
        var score = result.value;
        orig_dest_mode[score.origin.address] = 
          orig_dest_mode[score.origin.address] || 
          {};
        orig_dest_mode[score.origin.address][score.destination.address] = 
          orig_dest_mode[score.origin.address][score.destination.address] || 
          {};
        orig_dest_mode[score.origin.address][score.destination.address][score.mode.form] = 
          orig_dest_mode[score.origin.address][score.destination.address][score.mode.form] || 
          {
            origin: score.origin,
            destination: score.destination,
            mode: score.mode,
            score: score.score
          };
      }
      else {
        console.log('scorePromises allSettled:', result.error);
      }
    });

      //some business logic
      var out = {
        /* ... */
      };
      deferred.resolve(out);
  });
};
```

----

## Horizon

- [x] Rewrite the `Q` spaghetti in [walkre](https://github.com/bguiz/walkre)
  - Demonstrate how declaratively defining dependent queries can make code more comprehensible
- [x] Feature to reference results of dependent queries *inline* in query data
  - Kinda [like this](http://nmjenkins.com/presentations/network-speed.html#/17)
- [x] Separate [qryq](https://github.com/bguiz/qryq) into its own library - out of [walkre](https://github.com/bguiz/walkre)
- [x] Unit tests - `nodeunit`
- [x] Pick a licence for this library - GPL v3
- [ ] Infer `depends` if not provided using two passes when parsing each `qry`

---

### Farther Horizon

- [ ] Cyclic graph detection in dependent query queue validation
- [ ] Allow client to create promises and pass in to `qryq`
- [ ] Benchmarking for performance
- [ ] Load testing/ stress testing
  - Start including high latency ops, e.g. disk I/O
- [ ] Create a NodeJs/ ExpressJs server wrapper for `qryq`
  - For full stack load testing/ stress testing

----

## Fin

- Recommendations for load testing a NodeJs server?
- What other libraries are there out that that perform this function? In other languages?
- Submit some patches!

----

## Thank you

[bguiz.com](http://bguiz.com "my blog")

[@bguiz](http://twitter.com/bguiz "my twitter")

[bit.ly/qryq](http://bguiz.com/post/54620002947/qryq "qryq intro")

[github.com/bguiz/qryq](https://github.com/bguiz/qryq "qryq source")

[/doco/present/markdown/present.md](https://github.com/bguiz/qryq/blob/master/doco/present/markdown/present.md "this presentation")

[github.com/bguiz/walkre](https://github.com/bguiz/walkre "walkre source")

----

## The Code

----

## What is the `api` object?

- a JS object that gets passed into `qryq`
- each query in the queue names an `api`
- matches a function with the same name in the `api` object
- each function takes in a `deferred` and a `qry`
  - `qry`: input params
  - `deferred`: promise object, must call `reject` or `resolve`

```javascript
exports.api = {
  multiply: function(deferred, qry) {
    if (!qry || ! _.isNumber(qry.a) || ! _.isNumber(qry.b)) {
      deferred.reject('Must specify two params, a and b');
    }
    else {
      deferred.resolve(qry.a * qry.b);
    }
  }
}
```

----

## Test

var aQueryQueue = [
  {id: "q1", depends:[], api: "add", qry:{a:1, b:9}},
  {id: "q2", depends:[], api: "add", qry:{a:99, b:1}},
  {id: "q3", depends:['q1', 'q2'], api: "multiply", qry:{a: "#{q1}", b: "#{q2}"}},
  {id: "q4", depends:['q3'], api: "multiply", qry:{a: "#{q3}", b:5}}
];
var expectedResults = [
    { id: 'q1', value: 10 }, 
    { id: 'q2', value: 100 }, 
    { id: 'q3', value: 1000 }, 
    { id: 'q4', value: 5000 }
];

---

var deferred = Q.defer();
qryq.dependent(deferred, aQueryQueue, api);
deferred.promise.then(function(result) {
    test.ok(true, 'qryq resolved');
    console.log(JSON.stringify(result));
    test.ok(_.isArray(result), 'Returned and array');
    test.equals(result.length, 4, 'Returned 4 results');

    _.each(result, function(qryResult, idx) {
        test.ok(! _.isUndefined(qryResult.id));
        var response = qryResult.response;
        test.ok(_.isObject(response), 'Qry id='+qryResult.id+' has a response object');
        test.equal(qryResult.id, expectedResults[idx].id, 'Qry id='+qryResult.id+' has expected id');
        if (!! response) {
            test.equals(response.state, 'fulfilled', 'Qry id='+qryResult.id+' has expected state');
            test.equals(response.value, expectedResults[idx].value, 'Qry id='+qryResult.id+' has expected result');
        }
    }, this);
    test.done();
}, function(reason) {
    test.ok(false, 'qryq rejected, reason:'+JSON.stringify(reason));
    test.done();
});

----

## Parallel

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
var numApiCalls = qry.length;
var apiPromises = [];
_.each(qry, function(line) {
var apiQry = line.qry;
var apiName = line.api;
var apiFunc = api[apiName];
if (!apiFunc) {
  apiFunc = api.noSuchApi;
  apiQry = apiName;
}
apiPromises.push(async(apiFunc, apiQry));
});
  </code>
</pre>

---

### Parallel

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
Q.allSettled(apiPromises).then(function(apiResults) {
var out = [];
_.each(apiResults, function(apiResult, idx) {
  var result = _.extend({
    id: qry[idx].id,
    api: qry[idx].api},
    apiResult);
  out.push(result);
});
deferred.resolve(out);
});
  </code>
</pre>

----

## Sequential

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
var numApiCalls = qry.length;
var out = [];
function sequentialLine(idx) {
var line = qry[idx];
var apiQry = line.qry;
var apiName = line.api;
var apiFunc = api[apiName];
if (!apiFunc) {
  apiFunc = api.noSuchApi;
  apiQry = apiName;
}
var promise = async(apiFunc, apiQry);
promise.then(
  /* ... */
);
}
sequentialLine(0);
  </code>
</pre>

---

### Sequential

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

```javascript
promise.then(
  function(result) {
    out.push(result);
    if (idx < numApiCalls - 1) {
      sequentialLine(idx + 1);
    }
    else {
      deferred.resolve(out);
    }
  },
  function(err) {
    deferred.reject({
      error: 'Cannot process query '+apiQry.id,
      detail: err,
      incompleteResults: out
    });
  }
);
```

----

## Dependent

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
var linePromisesHash = {};
var linePromises = [];
_.each(qry, function(line) {
var apiQry = line.qry;
var apiName = line.api;
var apiFunc = api[apiName];
if (!apiFunc) {
  apiFunc = api.noSuchApi;
  apiQry = apiName;
}
var linePromise = dependentLine(line, apiFunc, linePromisesHash);
linePromises.push(linePromise);
linePromisesHash[line.id] = linePromise;
});
  </code>
</pre>

---

### Dependent

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
var dependentLine = function(line, apiFunc, linePromisesHash) {
  var lineDeferred = Q.defer();
  var dependsPromises = [];
  var depIds = line.depends;
  _.each(depIds, function(depId) {
    var dependPromise = linePromisesHash[depId];
    dependsPromises.push(dependPromise);
  });
  Q.allSettled(dependsPromises).then(function(dependsResults) {
    /* ... */
  });
  return lineDeferred.promise;
};
  </code>
</pre>

---

### Dependent

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
Q.allSettled(dependsPromises).then(function(dependsResults) {
var dependsResultsHash = {};
_.each(dependsResults, function(depResult, idx) {
  var depId = depIds[idx];
  if (depResult.state === 'fulfilled') {
    dependsResultsHash[depId] = depResult;
  }
  else {
    dependsResultsHash[depId] = null;
  }
});
var lineQryWithDepends = {};
_.extend(
  lineQryWithDepends,
  line.qry,
  {dependsResults: dependsResultsHash}
);
apiFunc(lineDeferred, lineQryWithDepends);
});
  </code>
</pre>

---

### Dependent

[github.com/bguiz/qryq](http://github.com/bguiz/qryq)

<pre>
  <code class="js">
Q.allSettled(linePromises).then(function(lineResults) {
var out = [];
_.each(lineResults, function(lineResult, idx) {
  var lineId = qry[idx].id;
  out.push({
    id: lineId,
    response: lineResult
  });
});
deferred.resolve(out);
});
  </code>
</pre>

----

==> qryq-0.0.9.tgz: package/doco/present/markdown/present.md <==
# qryq

## stop RESTing, start using query queues

<sup>
  /ˈkwərik/
</sup>

### Brendan Graetz

![Brendan Graetz](https://si0.twimg.com/profile_images/2697395233/8dec79da8f5963a4cad4da0ebd6a532d.png "Brendan Graetz on Twitter")

[@bguiz](http://bguiz.com "Brendan Graetz on Twitter")

[bguiz.com](http://bguiz.com "Brendan Graetz")

----

## In one sentence

`qryq` is a NodeJs library that allows one to express a series of queries and define dependencies between them either in parallel, in sequence, or in a directed acyclic graph.

![Directed Acyclic Graph](http://upload.wikimedia.org/wikipedia/commons/thumb/3/39/Directed_acyclic_graph_3.svg/356px-Directed_acyclic_graph_3.svg.png)

---

<blockquote>
A directed graph with no directed cycles.<br />
That is, it is formed by a collection of vertices and directed edges, each edge connecting one vertex to another, such that there is no way to start at some vertex v and follow a sequence of edges that eventually loops back to v again
</blockquote>

-- [Directed acyclic graph](http://en.wikipedia.org/wiki/Directed_acyclic_graph)

----

## Implementation

- [node.js](http://nodejs.org)
- [Q](https://github.com/kriskowal/q)
- [underscore.js](http://underscorejs.org)
- [nodeunit](https://github.com/caolan/nodeunit)

----

## Inspiration

- Neil Jenkin's talk Tips, Tricks and Hacks in the Pursuit of Speed
  - [REST is slow](http://nmjenkins.com/presentations/network-speed.html#/14)
  - [Concatenate requests](http://nmjenkins.com/presentations/network-speed.html#/15)
  - [Concatenate responses](http://nmjenkins.com/presentations/network-speed.html#/16)

---

<pre>
  <code class="js">
POST /api/

[
    [ 'deleteMessages', {
        idList: [ 'msg1' ]
    }],
    [ 'getMailboxMessageList', {
        mailboxName: 'Inbox',
        position: 0,
        limit: 30,
        sort: 'date descending'
    }]
]
  </code>
</pre>

---

## Inspiration

- [Play framework](http://playframework.com)'s
  - [Linkedin talk by Yevgeniy Brikman](http://www.slideshare.net/brikis98/the-play-framework-at-linkedin)
  - See slides 85 through 88

---

<div>
  <iframe src="http://www.slideshare.net/slideshow/embed_code/22423382?rel=0&startSlide=85" width="512" height="421" frameborder="0" marginwidth="0" marginheight="0" scrolling="no" style="border:1px solid #CCC;border-width:1px 1px 0;margin-bottom:5px" allowfullscreen webkitallowfullscreen mozallowfullscreen>
  </iframe>
  <div style="margin-bottom:5px">
    <strong> <a href="http://www.slideshare.net/brikis98/the-play-framework-at-linkedin" title="The Play Framework at LinkedIn" target="_blank">The Play Framework at LinkedIn</a></strong> from <strong><a href="http://www.slideshare.net/brikis98" target="_blank">Yevgeniy Brikman</a></strong>
  </div>
</div>

----

## The Itch

- NodeJs callback spaghetti
- Fix this using promises
- While better, if the code is sufficiently complex, you can still end up with:
- Promise spaghetti

![Callback Spaghetti](image/callback-spaghetti.png)

----

### Light Bulb

[Question on S/O](http://stackoverflow.com/questions/17342401/q-executing-a-series-of-promises-and-defining-dependencies-between-them-in-a-d "Q - executing a series of promises and defining dependencies between them in a DAG")
![Stackoverflow Question](image/stackoverflow-qn.png)

----

## The Query Queue

<pre>
  <code>
[
  {id: "q1", api: "add", qry:{a:1, b:9}},
  {id: "q2", api: "add", qry:{a:99, b:1}},
  {id: "q3", api: "multiply", qry:{a: "#{q1}", b: "#{q2}"}},
  {id: "q4", api: "multiply", qry:{a: "#{q3}", b:5}}
]
  </code>
</pre>

<pre>
  <code>
`q2`                          --> add(99, 1)          --> 100
`q1`                          --> add(1, 9)           --> 10
`q3` --> multiply(`q1`, `q2`) --> multiply(10, 100)   --> 1000
`q4` --> multiply(`q3`, 5)    --> multiply(1000, 5)   --> 5000
  </code>
</pre>

- `q1` and `q2` may execute in any order,
- `q3` may only execute after *both* `q1` and `q2`,
- and `q4` executes last.
- Wiring is done automatically by `qryq`

---

### What about async?

- Dev productivity in accomplishing the same thing:
  - Sequential: ~Same
  - Parallel: ~Same
  - Dependent: A lot easier

----

### Code

- Let's look at some!

- What is it like to write the same thing using async vs. using qryq?
- (also callbacks, and promises)

----

### Key comparisons

- assembly && execution
- declarative || imperative
- developer productivity
- what it is and what it isn't

----

## Benefits

- Developer productivity
- Bandwidth && latency savings

---

### Benefits - Dev Productivity

<blockquote>
This is the Unix philosophy: Write programs that do one thing and do it well. Write programs to work together.
</blockquote>

-- Doug McIlroy, invented pipes in UNIX

---

![Developer productivity](http://imgs.xkcd.com/comics/mu.png)

---

![Developer productivity](http://imgs.xkcd.com/comics/is_it_worth_the_time.png)

---

### Benefits - Dev Productivity

- Less need to write dedicated API endpoints
  - instead write small API endpoints, and chain them together
  - readable && composable
  - declarative query rather than imperative
  - avoid callback spaghetti && promise spaghetti

---

### Benefits - 'Net Traffic

- Concatenation of
  - Multiple requests
  - Multiple responses
- [Protocol overhead](http://sd.wareonearth.com/~phil/net/overhead/) minimised

---

### Benefits - Dev Productivity


- Less duplication of biz logic required
  - Client/ server
- Groups several queries together as an atomic unit
  - [Asynchronous UIs](http://blog.alexmaccaw.com/asynchronous-ui)

----

## Limitations

- non-RESTful
- Testability?
- Expression engine

---

### Limitations - REST

- Do you need REST when you are *not* doing CRUD?
- What if you typically chain more than one CRUD operation together?

---

### Limitations - Testing

- Harder because clients may compose API calls in novel ways
- Need to write more robust code

---

![Wisdom of the ancients - errors](http://imgs.xkcd.com/comics/wisdom_of_the_ancients.png)

---

### Limitations - Expressions

<blockquote>
`#{previousQry}.flights.length`
</blockquote>

- A `qry` references results of another `qry` in the same `qryq`
- Limited: Can only "drill down" through properties

----

### Where to use: Client side

- Instead of making multiple XHR requests to each RESTful API endpoints
  - and assemble them together by hand
- Invoke qryq, and it will assemble the multiple requests together itself
  - still makes multiple XHR requests
- Great if goal is not so much bandwidth savings
  - instead dev productivity
  - (or if you're *not* making any XHR requests)

----

### Where to use: Server side

- Instead of making multiple XHR requests to each RESTful API endpoints
  - and assemble them together by hand on the client
- Invoke qryq API endpoint, and it assembles and executes the multiple requests
  - Single XHR request-response pair
- Dev productivity + Bandwidth savings
  - Only if your server is in JS <sup>&lowast;</sup>

----

### How do I use qryq?

- Instructions in the main [README.md](https://github.com/bguiz/qryq)

----

## Horizon

- [x] Rewrite the `Q` spaghetti in [walkre](https://github.com/bguiz/walkre)
  - Demonstrate how declaratively defining dependent queries can make code more comprehensible
- [x] Feature to reference results of dependent queries *inline* in query data
  - Kinda [like this](http://nmjenkins.com/presentations/network-speed.html#/17)
- [x] Separate [qryq](https://github.com/bguiz/qryq) into its own library - out of [walkre](https://github.com/bguiz/walkre)
- [x] Unit tests - `nodeunit`
- [x] Pick a licence for this library - GPL v3
- [x] Infer `depends` if not provided using two passes when parsing each `qry`
- [x] Comparison with async, use diamond shaped query graph

---

![Goto - how bad can it be?](http://imgs.xkcd.com/comics/goto.png)

---

### Farther Horizon

- [ ] Cyclic graph detection in dependent query queue validation
- [ ] Allow client to create promises and pass in to `qryq`
- [ ] Benchmarking for performance
- [ ] Load testing/ stress testing
  - Start including high latency ops, e.g. disk I/O
- [ ] Create a NodeJs/ ExpressJs server wrapper for `qryq`
  - For full stack load testing/ stress testing
- [ ] Minify
- [ ] Restructure to follow [CommonJS directory structure](https://npmjs.org/doc/json.html#directories)
- [ ] Publish as a bower project
- [ ] Write test cases for series and paralllel

----

### Decisions

- Should the server return all reponses?
  - Perhaps allow specifying which queries we are interested in the return value of.
- Port to other languages?
  - Which ones?
- Should remove sequential and parallel altogether?
    - Just focus on dependent (DAG) instead?

---

![Supported Features](http://imgs.xkcd.com/comics/supported_features.png)

----

## Fin

- Recommendations for load testing a NodeJs server?
- What other libraries are there out that that perform this function? In other languages?
- Submit some patches!

----

## Thank you

[bguiz.com](http://bguiz.com "my blog")

[@bguiz](http://twitter.com/bguiz "my twitter")

[github.com/bguiz/qryq](https://github.com/bguiz/qryq "qryq source")
